//  Needs the following defines
// take these for standard values
/*
// Names of the Input Pins of the LED Driver according to the datasheet
define VPRG  PIN_D0  //27
define SIN_PIN   PIN_D1  //26 in the DIP package
define SCLK  PIN_D2  //25
define XLAT  PIN_D3  //24
define BLANK PIN_D4  //23
define DCPRG PIN_D5  //19
define GSCLK PIN_D6  //18
define XERR  PIN_D7  //16

define CHANNELS 16 // set this to a multiple of 16 if you cascade more than one TLC5940
*/

int dcs[CHANNELS]; // dot correction values
//long gss[CHANNELS]; // grey scale values

// Sets the 6bit "Dot Correction" of all channels
// see datasheet page 12 and 13
void setDC(int dc[]) {
   int i, j, tmp;

   output_low(SCLK);
   output_high(VPRG);

   for (i = 0; i < CHANNELS; i++) {
      tmp = dc[(CHANNELS - 1) - i];
      for (j = 0; j < 6; j++) {
         if (tmp & 0x20) {
            output_high(SIN_PIN);

         }
         else {
            output_low(SIN_PIN);
         }
         output_high(SCLK);
         tmp <<= 1;
         output_low(SCLK);
      }
   }

   output_high(XLAT);
   output_low(XLAT);
}

// sets the Greyscale Values of all channels
// first: adds an additional SCLK pulse (see datasheet page 14
void setGS(int first) { // (long gs[], int first) {
   int i, j;
   long tmp;

   output_low(SCLK);
   output_low(VPRG);

   for (i = 0; i < CHANNELS; i++) {
      tmp = 0xffff; //gs[(CHANNELS - 1) - i];
      for (j = 0; j < 12; j++) {
         if (tmp & 0x800) {
            output_high(SIN_PIN);
         }
         else {
            output_low(SIN_PIN);
         }
         output_high(SCLK);
         tmp <<= 1;
         output_low(SCLK);
      }
   }

   output_high(XLAT);
   output_low(XLAT);

   if (first) {
       output_high(SCLK);
       output_low(SCLK);
   }
}

// initializes the GS PWM operation (not fully implemented now)
// see datasheet page 15
void initGS() {
   int i, j;
   long tmp;

   output_low(GSCLK);

   output_high(BLANK);
   output_low(BLANK);

   output_high(GSCLK);
   output_low(GSCLK);
}

void initTLC5940(void) {
   long i;


/*   input(SIN_PIN);
   input(SCLK);
   input(XLAT);
   input(VPRG);
   input(DCPRG); // take DC values from the registers (not EEPROM) see datasheet page 12
   input(GSCLK);
   input(BLANK);
   delay_ms(1);
*/
   // initialize all the input pins of the LED driver
   output_low(SIN_PIN);
   output_low(SCLK);
   output_low(XLAT);
   output_low(VPRG);
   output_high(DCPRG); // take DC values from the registers (not EEPROM) see datasheet page 12
   output_low(GSCLK);
   output_low(BLANK);

   delay_ms(1); // unnecessary?
       // initialisiere DC und GS
   /*for (i = 0; i < CHANNELS; i++) {
      dcs[i] = (i % 2) ? 0 : 63;
      //gss[i] = 1024;
   }*/
   setDC(dcs);
   setGS(true);
   initGS();
}

void refreshDisplay(void) {
   setDC(dcs);
}

void clearDisplay(void) {
   long i;

   for (i = 0; i < CHANNELS; i++) {
      dcs[i] = 0;
   }
   refreshDisplay();
}

