/******************************************************
Demo Anwendung fuer:
TLC5940: 16 Channel LED Driver w/DOT Correction & Grayscale PWM Control
http://focus.ti.com/docs/prod/folders/print/tlc5940.html

realesed under the GPL by Philipp Tiefenbacher
******************************************************/
#include "18f458.h"
//#include "16f877.h"

#device ADC=10
#include <STDLIB.H>
#include <math.h>
#fuses hs,nowdt,put,noprotect
#use delay(clock=20000000)	//20MHz, fuer delay()


// Names of the Input Pins of the LED Driver according to the datasheet
#define VPRG  PIN_D1  //27
#define SIN_PIN   PIN_D2  //26 in the DIP package
#define SCLK  PIN_D3  //25
#define XLAT  PIN_D4  //24
#define BLANK PIN_D5  //23
#define DCPRG PIN_D6  //19
#define GSCLK PIN_D7  //18

#define CHANNELS 64 // set this to a multiple of 16 if you cascade more than one TLC5940

#include "TLC5940.h"

#use rs232(baud=57600, xmit=PIN_C6, rcv=PIN_C7, Errors)


#define PHONE_GREEN_0 0
#define PHONE_GREEN_1 1
#define PHONE_GREEN_2 2
#define PHONE_GREEN_3 3

#define ATOM_RED_0 4
#define ATOM_RED_1 5
#define ATOM_RED_2 6
#define ATOM_RED_3 7
#define ATOM_RED_4 8
#define ATOM_RED_5 9

#define PHONE_BLUE_0 10
#define PHONE_BLUE_1 11
#define PHONE_BLUE_2 12


int atom_state, atom_dir;
int phone_state;
long phone_count;


void init_metalab() {
   atom_state = 0;
   atom_dir = 1;
   phone_state = 0;
   phone_count = 0;
}

void step_atom() {
   atom_state += atom_dir;

   if (atom_state == 63 || atom_state == 0) {
      atom_dir = -atom_dir;
   }

   dcs[ATOM_RED_0] = atom_state;
   dcs[ATOM_RED_1] = atom_state;
   dcs[ATOM_RED_2] = atom_state;
   dcs[ATOM_RED_3] = atom_state;
   dcs[ATOM_RED_4] = atom_state;
   dcs[ATOM_RED_5] = atom_state;
}

void step_phone() {
   phone_count++;
   switch (phone_state) {
      case 0:
         if (phone_count < 100) {
            dcs[PHONE_BLUE_0] = 63;
            dcs[PHONE_BLUE_1] = 63;
            dcs[PHONE_BLUE_2] = 63;
         }
         else if (phone_count >= 100 && phone_count <= 163) {
            if ((phone_count & 1) == 1) {
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
            else {
               dcs[PHONE_BLUE_0] = 163 - phone_count;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }

         if (phone_count >= 164 && phone_count <= 227) {
            dcs[PHONE_GREEN_0] = 63;
            dcs[PHONE_GREEN_1] = 63;
            dcs[PHONE_GREEN_2] = 63;
            dcs[PHONE_GREEN_3] = 63;

            dcs[PHONE_BLUE_1] = 227 - phone_count;
            dcs[PHONE_BLUE_2] = 227 - phone_count;
         }

         if (phone_count > 227) {
            phone_count = 0;
         }
         break;
   }

}


void main ()
{
   int i, count;
   int color, index;
   int key;

   initTLC5940();

   init_metalab();

   count = 0;
   color = 0;

   for (i=0; i < 64; i++) {
         dcs[i]=0;
   }

	while (1) {

      if (kbhit()) {
         key = getch();
         if (key == 'r') {
            color = 16;
         }
         if (key == 'g') {
            color = 32;
         }
         if (key == 'v') {
            color = 48;
         }
         if (key == 'x') {
            color = 0;
         }
         else if (key >= '0' && key <= '9') {
            index = key - '0';
            index += color;
            dcs[index] = 63 - dcs[index];
         }
         else if (key >= 'a' && key <= 'f') {
            index = 10 + key - 'a';
            index += color;
            dcs[index] = 63 - dcs[index];
         }
      }

      refreshDisplay();
      delay_ms(50);
	}
}
