/******************************************************
Demo Anwendung fuer:
TLC5940: 16 Channel LED Driver w/DOT Correction & Grayscale PWM Control
http://focus.ti.com/docs/prod/folders/print/tlc5940.html

realesed under the GPL by Philipp Tiefenbacher
******************************************************/
#include "18f458.h"
//#include "16f877.h"

#device ADC=10
#include <STDLIB.H>
#include <math.h>
#fuses hs,nowdt,put,noprotect
#use delay(clock=20000000)	//20MHz, fuer delay()


// Names of the Input Pins of the LED Driver according to the datasheet
#define VPRG  PIN_D1  //27
#define SIN_PIN   PIN_D2  //26 in the DIP package
#define SCLK  PIN_D3  //25
#define XLAT  PIN_D4  //24
#define BLANK PIN_D5  //23
#define DCPRG PIN_D6  //19
#define GSCLK PIN_D7  //18

#define CHANNELS 64 // set this to a multiple of 16 if you cascade more than one TLC5940

#include "TLC5940.h"

#use rs232(baud=57600, xmit=PIN_C6, rcv=PIN_C7, Errors)


#define PHONE_GREEN_0 0
#define PHONE_GREEN_1 1
#define PHONE_GREEN_2 2
#define PHONE_GREEN_3 3

#define ATOM_RED_0 4
#define ATOM_RED_1 5
#define ATOM_RED_2 6
#define ATOM_RED_3 7
#define ATOM_RED_4 8
#define ATOM_RED_5 9

#define PHONE_BLUE_0 10
#define PHONE_BLUE_1 11
#define PHONE_BLUE_2 12

#define CHANGE 5



int atom_state, atom_dir, atom_delay;
int phone_state;
long phone_count;


void init_metalab() {
   atom_state = 0;
   atom_dir = 1;
   atom_delay = 120;
   phone_state = 0;
   phone_count = 0;
}

void step_atom() {
   if (atom_delay > 0) {
      atom_delay--;
   }
   else {
      atom_state += atom_dir;

      if (atom_state == 63 || atom_state == 0) {
         atom_dir = -atom_dir;
      }

      if (atom_state == 0) {
         atom_delay = 40;
      }



   }

   dcs[ATOM_RED_0] = atom_state;
   dcs[ATOM_RED_1] = atom_state;
   dcs[ATOM_RED_2] = atom_state;
   dcs[ATOM_RED_3] = atom_state;
   dcs[ATOM_RED_4] = atom_state;
   dcs[ATOM_RED_5] = atom_state;
}

#define BLUE_PHASE_LEN 300
#define BLINK_LEN 64
#define BLINK2_LEN 64
#define GREEN_PHASE_START (BLUE_PHASE_LEN+BLINK_LEN+BLINK2_LEN)
#define GREEN_PHASE_LEN 300
#define GREEN_BLINK_START (GREEN_PHASE_START+GREEN_PHASE_LEN)
#define GREEN_BLINK2_START (GREEN_PHASE_START+GREEN_PHASE_LEN+BLINK2_LEN)



void step_phone() {
   phone_count++;
   switch (phone_state) {
      case 0:
         if (phone_count < BLUE_PHASE_LEN) {
            dcs[PHONE_BLUE_0] = 63;
            dcs[PHONE_BLUE_1] = 63;
            dcs[PHONE_BLUE_2] = 63;
            dcs[PHONE_GREEN_1] = 0;
            dcs[PHONE_GREEN_0] = 0;
            dcs[PHONE_GREEN_1] = 0;
            dcs[PHONE_GREEN_2] = 0;
            dcs[PHONE_GREEN_3] = 0;
         }
         else if (phone_count < BLUE_PHASE_LEN+BLINK_LEN) {
            if (((phone_count-BLUE_PHASE_LEN)>>4 & 1) == 0) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }
         else if (phone_count < BLUE_PHASE_LEN+BLINK_LEN+BLINK2_LEN) {
            if (((phone_count-(BLUE_PHASE_LEN+BLINK_LEN))>>1 & 1) == 1) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }
         else if (phone_count < GREEN_BLINK_START) {
            dcs[PHONE_BLUE_0] = 0;
            dcs[PHONE_BLUE_1] = 0;
            dcs[PHONE_BLUE_2] = 0;
            dcs[PHONE_GREEN_0] = 63;
            dcs[PHONE_GREEN_1] = 63;
            dcs[PHONE_GREEN_2] = 63;
            dcs[PHONE_GREEN_3] = 63;
         }
         else if (phone_count < GREEN_BLINK_START+BLINK_LEN) {
            if (((phone_count-GREEN_BLINK_START)>>4 & 1) == 0) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }
         else if (phone_count < GREEN_BLINK_START+BLINK_LEN+BLINK2_LEN) {
            if (((phone_count-364)>>1 & 1) == 0) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }

         if (phone_count > GREEN_BLINK2_START+BLINK2_LEN) {
            phone_count = 0;
         }
         break;
   }

}


void main ()
{
   long i, j, count, count2;
   int start, gray, delay, flashdir;
   int now[CHANNELS], future[CHANNELS], blink[CHANNELS], blink_count[CHANNELS];
   int color, index, keyflash;
   int key;

   for (i=0; i < CHANNELS; i++) {
         dcs[i]=0;
         now[i] = 0;
         future[i] = 0;
         blink[i] = 0;
         blink_count[i] = 0;
   }

   initTLC5940();

   init_metalab();

   count = 0;
   count2 = 10;
   color = 0;
   delay = 1;
   flashdir = 1;
   
   puts("metalab rulez!");

	while (1) {
      count++;
      if (count >= 64) {
         for (i=0; i < CHANNELS; i++) {
            now[i] = future[i];
         }
         delay = rand() % 40; + 10;
         for (j = 0; j < 2; j++) {
            start = rand()%(CHANNELS-CHANGE+1);
            gray = rand() % 64;
            for (i = start; i < start+CHANGE; i++) {
               future[i] = gray;
            }

            start = rand()%(CHANNELS-CHANGE+1);
            gray = (rand() % 30) + 8;
            for (i = start; i < start+CHANGE; i++) {
               //blink[i] = gray;
            }

            start = rand()%(CHANNELS-CHANGE+1);
            for (i = start; i < start+CHANGE; i++) {
               future[i] = 0;
            }
         }

         count = 0;
         count2--;
         if (count2 == 0) {
            for (i = 0; i < CHANNELS; i++) {
               future[i] = 0;
               blink[i] = 0;
            }
            count2 = 10 + rand()%60;
            initTLC5940();
            puts("resettin");
         }

         flashdir = rand()%2;
      }

      for (i = 0; i < CHANNELS; i++) {
         if (blink[i] != 0 && blink_count[i] < blink[i]/2) {
            dcs[i] = 0;
         }
         else{
            dcs[i] = (now[i]*(64l-count)+future[i]*count)/64l;
         }
         if (blink[i] != 0) {
            blink_count[i]++;
            if (blink_count[i] >= blink[i]) {
               blink_count[i] = 0;
            }
         }
      }


      if (count < 24 && (count2%6) == 0) {
         if (count < 16) {
            i = count/2;
         }
         else {
            i = count-16;
            i = 7-i;
         }

         if (flashdir) {
            i = 7 - i;
         }
         i = 7 - i;

         dcs[i+16] = 0;
         dcs[i+32] = 0;
         dcs[i+48] = 0;

         dcs[31-i] = 0;
         dcs[47-i] = 0;
         dcs[63-i] = 0;
      }

      if (count < 8 && count2 == 10) {
         i = count;
         if (flashdir) {
            i = 7 - i;
         }

         dcs[i+16] = 63;
         dcs[i+32] = 63;
         dcs[i+48] = 63;

         dcs[31-i] = 63;
         dcs[47-i] = 63;
         dcs[63-i] = 63;
      }

      if (keyflash != 0) {
         keyflash--;
         i = keyflash;

         if (flashdir) {
            i = 7 - i;
         }

         dcs[i+16] = 63;
         dcs[i+32] = 63;
         dcs[i+48] = 63;

         dcs[31-i] = 63;
         dcs[47-i] = 63;
         dcs[63-i] = 63;
      }

      if (count == 23 && count2 == 23) {
         for (i = 0; i < CHANNELS; i++) {
            dcs[i] = 63;
         }
      }

      if (kbhit()) {
         switch (getch()) {
            case 'r':
               count = 0;
               for (i = 16; i < CHANNELS; i++) {
                  future[i] = 0;
                  now[i] = dcs[i];
               }
               for (i = 0; i < 16; i++) {
                  future[16 + i] = 63;
               }
               break;

            case 'g':
               count = 0;
               for (i = 16; i < CHANNELS; i++) {
                  future[i] = 0;
                  now[i] = dcs[i];
               }
               for (i = 0; i < 16; i++) {
                  future[32 + i] = 63;
               }
               break;

            case 'b':
               count = 0;
               for (i = 16; i < CHANNELS; i++) {
                  future[i] = 0;
                  now[i] = dcs[i];
               }
               for (i = 0; i < 16; i++) {
                  future[48 + i] = 63;
               }
               break;

            case 'X':
               for (i = 0; i < 16; i++) {
                  future[i] = 0;
               }
               break;

            case 'C':
               for (i = 16; i < CHANNELS; i++) {
                  future[i] = 0;
               }
               break;

            case 'x':
               for (i = 0; i < 16; i++) {
                  dcs[i] = 63;
               }
               break;

            case 'c':
               for (i = 16; i < CHANNELS; i++) {
                  dcs[i] = 63;
               }
               break;

            case ' ':
               keyflash = 8;
               break;
         }
      }


      step_atom();
      step_phone();

      refreshDisplay();
      delay_ms(20);
      //putc(65);
	}
}
