/******************************************************
Demo Anwendung fuer:
TLC5940: 16 Channel LED Driver w/DOT Correction & Grayscale PWM Control
http://focus.ti.com/docs/prod/folders/print/tlc5940.html

realesed under the GPL by Philipp Tiefenbacher
******************************************************/
#include "18f458.h"
//#include "16f877.h"

#device ADC=10
#include <STDLIB.H>
#include <math.h>
#fuses hs,nowdt,put,noprotect
#use delay(clock=20000000)	//20MHz, fuer delay()


// Names of the Input Pins of the LED Driver according to the datasheet
#define VPRG  PIN_D1  //27
#define SIN_PIN   PIN_D2  //26 in the DIP package
#define SCLK  PIN_D3  //25
#define XLAT  PIN_D4  //24
#define BLANK PIN_D5  //23
#define DCPRG PIN_D6  //19
#define GSCLK PIN_D7  //18

#define CHANNELS 64 // set this to a multiple of 16 if you cascade more than one TLC5940

#include "TLC5940.h"

#use rs232(baud=19200, xmit=PIN_C6, rcv=PIN_C7, Errors)


#define PHONE_GREEN_0 0
#define PHONE_GREEN_1 1
#define PHONE_GREEN_2 2
#define PHONE_GREEN_3 3

#define ATOM_RED_0 4
#define ATOM_RED_1 5
#define ATOM_RED_2 6
#define ATOM_RED_3 7
#define ATOM_RED_4 8
#define ATOM_RED_5 9

#define PHONE_BLUE_0 10
#define PHONE_BLUE_1 11
#define PHONE_BLUE_2 12


int atom_state, atom_dir, atom_delay;
int phone_state;
long phone_count;


void init_metalab() {
   atom_state = 0;
   atom_dir = 1;
   atom_delay = 120;
   phone_state = 0;
   phone_count = 0;
}

void step_atom() {
   if (atom_delay > 0) {
      atom_delay--;
   }
   else {
      atom_state += atom_dir;
   
      if (atom_state == 63 || atom_state == 0) {
         atom_dir = -atom_dir;
      }
      
      if (atom_state == 0) {
         atom_delay = 40;
      }
      
   
      dcs[ATOM_RED_0] = atom_state;
      dcs[ATOM_RED_1] = atom_state;
      dcs[ATOM_RED_2] = atom_state;
      dcs[ATOM_RED_3] = atom_state;
      dcs[ATOM_RED_4] = atom_state;
      dcs[ATOM_RED_5] = atom_state;
   }
}

void step_phone() {
   phone_count++;
   switch (phone_state) {
      case 0:
         if (phone_count < 100) {
            dcs[PHONE_BLUE_0] = 63;
            dcs[PHONE_BLUE_1] = 63;
            dcs[PHONE_BLUE_2] = 63;
         }
         else if (phone_count >= 100 && phone_count <= 163) {
            if ((phone_count>>4 & 1) == 1) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }

         if (phone_count >= 164 && phone_count <= 227) {
         if ((phone_count>>2 & 1) == 1) {
               dcs[PHONE_GREEN_0] = 63;
               dcs[PHONE_GREEN_1] = 63;
               dcs[PHONE_GREEN_2] = 63;
               dcs[PHONE_GREEN_3] = 63;
               dcs[PHONE_BLUE_0] = 0;
               dcs[PHONE_BLUE_1] = 0;
               dcs[PHONE_BLUE_2] = 0;
            }
            else {
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_0] = 0;
               dcs[PHONE_GREEN_1] = 0;
               dcs[PHONE_GREEN_2] = 0;
               dcs[PHONE_GREEN_3] = 0;
               dcs[PHONE_BLUE_0] = 63;
               dcs[PHONE_BLUE_1] = 63;
               dcs[PHONE_BLUE_2] = 63;
            }
         }

         if (phone_count > 227) {
            phone_count = 0;
         }
         break;
   }

}

void render_kitt(int offset, float pos, float dir, float maxDist) {
   float dist;
   int i;

   for (i = 0; i < 8; i++) {
      if (i < pos) dist = pos - (i+0.5);
      else dist = i - pos;

      //if (dist >= 4.0) {
      //   dist = 8.0 - dist;
      //}

      if (dist < 1.0) {
         dist = 0.0;
      }
      if (dist <= maxDist) {
         dcs[offset + i] = 63 - ((63.0/(maxDist*maxDist)) * dist*dist);
         dcs[offset + 8 + (7-i)] = 63 - ((63.0/(maxDist*maxDist)) * dist*dist);
      }
      else {
         dcs[offset + i] = 0;
         dcs[offset + 8 + (7-i)] = 0;
      }
   }
}


void render_flash(int offset, long count) {
   int i;

   if (count < (long)64) {
      for (i = 16; i <64; i++) {
         dcs[i] = 0;
      }
      for (i = 0; i <16; i++) {
         dcs[offset + i] = 63 - count;
      }
   }
}

void render_ball(int offset, float pos) {
   float dist;
   int i;

   for (i = 0; i < 16; i++) {
      if (i < pos) dist = pos - i;
      else dist = i - pos;

      if (dist >= 8.0) {
         dist = 16 - dist;
      }

      if (dist < 1.0) {
         dist = 0.0;
      }
      if (dist <= 2.0) {
         dcs[offset + i] = 63 - ((63.0/4.0) * dist*dist);
      }
      else dcs[offset + i] = 0;
   }
}

#define BALLS 2
#define FLASHES 3
#define KITTS 2

void main ()
{
   int i, count;
   int color;
   float pos[BALLS];
   float dir[BALLS];
   int offset[BALLS];
   float factor;
   
   long flash_count[FLASHES];
   long flash_max[FLASHES];
   int flash_offset[FLASHES];

   int kitt_offset[KITTS];
   float kitt_pos[KITTS];
   float kitt_dir[KITTS];
   
   kitt_offset[0] = 48;
   kitt_pos[0] = 0.1;
   kitt_dir[0] = 0.2;


   kitt_offset[1] = 32;
   kitt_pos[1] = 7.9;
   kitt_dir[1] = -0.2;

/*
   kitt_offset[2] = 32;
   kitt_pos[2] = 7.8;
   kitt_dir[2] = 0.11;

   kitt_offset[3] = 48;
   kitt_pos[3] = 7.9;
   kitt_dir[3] = 0.11;*/


   pos[0] = 0.0;
   dir[0] = -0.15;
   offset[0] = 16;
  ///*
   //pos[1] = 0.0;
   //dir[1] = 0.2;
   //offset[1] = 32;

   pos[1] = 0.0;
   dir[1] = 0.1;
   offset[1] = 48;
//*/

   flash_count[0] = 0;
   flash_max[0] = 600;
   flash_offset[0] = 16;

   flash_count[1] = 200;
   flash_max[1] = 600;
   flash_offset[1] = 32;

   flash_count[2] = 400;
   flash_max[2] = 600;
   flash_offset[2] = 48;

   initTLC5940();

   init_metalab();

   count = 0;
   color = 0;

   for (i=0; i < 64; i++) {
      dcs[i]=0;
   }

	while (1) {

      for (i = 0; i < BALLS; i++) {
         pos[i] += dir[i];
         if (pos[i] >=   16.0) {
            pos[i] -= 16.0;
         }
         if (pos[i] < 0.0) {
            pos[i] += 16.0;
         }
         //render_ball(offset[i], pos[i]);
      }

      for (i = 0; i < FLASHES; i++) {
         flash_count[i]++;
         if (flash_count[i] > flash_max[i]) {
            flash_count[i] = 0;
         }
         //render_flash(flash_offset[i], flash_count[i]);
      }

      for (i = 0; i < KITTS; i++) {
         
         if (kitt_pos[i] < 0.5 || kitt_pos[i] > 7.5) {
            factor = 0.25;
         }
         else {
            factor = 1.0;
         }
         kitt_pos[i] += kitt_dir[i] * factor;
         if (kitt_pos[i] > 8.0) {
            kitt_dir[i] = kitt_dir[i] * (-1.0);
            kitt_pos[i] = 8.0;
         }
         if (kitt_pos[i] < 0.0) {
            kitt_dir[i] = kitt_dir[i] * (-1.0);
            kitt_pos[i] = 0.0;
         }
         render_kitt(kitt_offset[i], kitt_pos[i], kitt_dir[i], 2.0);
      }

      step_atom();
      step_phone();
      refreshDisplay();
      //puts("send!");
      delay_ms(20);
	}
}
